import random
import math

import cocos
import pyglet
import pymunk
from pymunk.vec2d import Vec2d
from pyglet.gl import *

import Actor
import Static

BLOB_COLLISION_TYPE = 1
DEFAULT_COLLISION_TYPE = 0

def make_joint(circles, ia, ib):
   bodya = circles[ia][0]
   bodyb = circles[ib][0]
   joint = pymunk.PinJoint(bodya, bodyb, (0,0), (0,0))
   return joint

def make_spring(circles, ia, ib):
   bodya = circles[ia][0]
   bodyb = circles[ib][0]
   length = Vec2d(bodyb.position-bodya.position).get_length()
   spring = pymunk.DampedSpring(bodya, bodyb, (0,0), (0,0), length, 15000, 6.0)
   return spring

class Blob(Actor.Actor):


    def __init__(self, context, pos, blob_type = "Blob"):
        super(Blob, self).__init__(pos)
        group = self.id + 3

        static_body = pymunk.Body(pymunk.inf, pymunk.inf)

        cornercount = 10
        mainradius = 15

        circles = []
        corners = []
        for i in range(cornercount):
            ang = ((2*math.pi)/cornercount)*i
            corner = Vec2d(0,mainradius)
            corner.rotate(ang)
            corners.append(corner)

        for corner in corners:
            radius = 6
            mass = 10
            #to make more mass in the bottom than top of blob use this instead:
            #mass = 20 - (mainradius + corner.y) / float(mainradius) * 5
            moment = pymunk.inf # no rotation
            body = pymunk.Body(mass, moment)
            body.position = Vec2d(corner) + pos
            shape = pymunk.Circle(body, radius)
            shape.friction = 0.5
            shape.elasticity = 0.0
            shape.collision_type = BLOB_COLLISION_TYPE
            #shape.surface_velocity = (100,500)
            shape.group = group
            shape.glue_is_active = False
            shape.actor = self
            context.space.add(body, shape)
            circle = [body, shape]
            circles.append(circle)
            body.x = corner
            # motor = pymunk.SimpleMotor(static_body, body, -20)
            # motor.max_bias = 10000
            # context.space.add(motor)

        self.joints = []
        self.springs = []
        for i in range(cornercount):
            j = (i-1) % cornercount
            joint = make_joint(circles, i, j)
            self.joints.append(joint)
            context.space.add(joint)

            n = int(cornercount*0.4)
            s = (i+n) % cornercount
            spring = make_spring(circles, i, s)
            context.space.add(spring)
            self.springs.append(spring)

        self.circles = circles

        self.glue_joints = []
        # usch :D
        self.context = context
        self.test = 1
        self.center = Vec2d(self.x, self.y)

        self.fillcolor = (0,1,0)
        self.edgecolor = (0,1,0)
        self.blob_type = blob_type
        self.set_type(context, blob_type)
        self.prevPos = Vec2d(self.x, self.y)
        self.touching = 0
        self.switchtimer = 0

        context.space.add_collision_handler(DEFAULT_COLLISION_TYPE, DEFAULT_COLLISION_TYPE, self.glue_callback, None, None, None)
        context.space.add_collision_handler(BLOB_COLLISION_TYPE, DEFAULT_COLLISION_TYPE, self.glue_callback, None, None, None)
        context.space.add_collision_handler(BLOB_COLLISION_TYPE, BLOB_COLLISION_TYPE, self.glue_callback, None, None, None)

    def update(self, context):
        self.x = self.circles[0][0].position.x
        self.y = self.circles[0][0].position.y
        self.x, self.y = self.getcenter()

        joints_to_remove = set()

        for joint in self.glue_joints:
            d = (joint.a.position + joint.anchr1).get_distance(joint.b.position + joint.anchr2)
            if d > 3:
                joints_to_remove.add(joint)

        for joint in joints_to_remove:
            try:
                joint.glue_actors.remove(self)
                if joint.glue_actors == []:
                    context.space.remove(joint)
                self.glue_joints.remove(joint)
                #print "removeing", joint
            except:
                print "didn't remove joint %s" % self.blob_type

        if self.blob_type == "MoveBlob":
            xspeed = self.x - self.prevPos.x
            self.sprite.opacity = 150
            #self.sprite.rotation = 0
            if self.touching > 0:
                self.sprite.opacity = 250
                if xspeed != 0:
                    if xspeed < 0:
                        self.sprite.rotation = 180
                    else:
                        self.sprite.rotation = 0
                    xspeed /= abs(xspeed)

                    for body, shape in self.circles:
                        body.apply_impulse(Vec2d(xspeed * 50, 0))


        if self.blob_type == "JumpBlob" and self.switchtimer < 0.0:
            self.sprite.opacity = 50
            yspeed = self.y - self.prevPos.y
            if yspeed < 0:
                if self.touching > 0.2:
                    self.touching = 0.3
                    self.sprite.opacity = 250
                    for body, shape in self.circles:
                        body.apply_impulse(Vec2d(0,5000))

        self.prevPos = Vec2d(self.x, self.y)
        self.touching -= 0.2
        if self.touching < 0:
            self.touching = 0

        if self.sprite.scale > 1:
            self.sprite.scale -= 0.05

        self.switchtimer -= context.dt

    def getcenter(self):
        self.test = 1 - self.test
        if self.test:
            self.center = Vec2d(0,0)
            for c in self.circles[::2]:
               self.center += c[0].position
            self.center = self.center / (len(self.circles)/2)
        return self.center

    def draw(self):
        #return
        pyglet.gl.glPushMatrix()
        #self.transform()

        #glColor3f(1,1,1)

        edge_points = sum(map(lambda c: tuple(c[0].position), self.circles), ())
        len_edge_points = len(edge_points)/2
        is_ = []
        for i in range(1,len_edge_points-1):
            is_.append(0)
            is_.append(i)
            is_.append(i+1)
        cs = self.fillcolor * len_edge_points

        vs = edge_points
        vs = [c[0].position + (c[0].position-self.getcenter()) * 0.4 for c in self.circles]
        vs = sum([(p.x, p.y) for p in vs],())

        #print ps
        pyglet.graphics.draw_indexed(len_edge_points, pyglet.gl.GL_TRIANGLES, is_,
                            ('v2f', vs),
                              ('c3f', cs))
        #print edge_points
        vs = edge_points
        vs = [c[0].position + (c[0].position-self.getcenter()) * 0.45 for c in self.circles]
        vs = sum([(p.x, p.y) for p in vs],())
        vs = vs[-2:] + vs[-2:] + vs + vs[-2:]
        cs = self.edgecolor * (len_edge_points+3)


        #pyglet.gl.glLineWidth(3)
        pyglet.graphics.draw(len(vs)/2, pyglet.gl.GL_LINE_STRIP,
                             ('v2f', vs),
                               ('c3f', cs))

        pyglet.gl.glPopMatrix()

    def get_drag_object(self, p):
        drag_body = pymunk.Body(pymunk.inf,pymunk.inf)
        drag_body.position = p

        springs = []

        for body, _ in self.circles:
            rest_length = drag_body.position.get_distance(body.position)
            ds = pymunk.DampedSpring(drag_body, body, (0,0), (0,0), rest_length, 1000, 15)
            self.context.space.add(ds)
            springs.append(ds)
        def detach_func(body):
            self.context.space.remove(springs)
        return Actor.DragObject(drag_body, detach_func)



    def moveRel(self, x, y):
        for body, shape in self.circles:
            body.position.x += x
            body.position.y += y

    def move(self, x, y):
        x = x - self.circles[0][0].position.x
        y = y - self.circles[0][0].position.y
        for body, shape in self.circles:
            body.position.x += x
            body.position.y += y

    def set_speed(self, speed):
        for c in self.circles:
            c[1].surface_velocity = (speed,0)

    def glue_callback(self, space, arbiter, *args, **kwargs):
        #self.touching = 1
        #self.sprite.scale = 1.5
        shapea, shapeb = arbiter.shapes


        if isinstance(shapea.actor, Blob) and  isinstance(shapeb.actor, Static.Static):
            shapea.actor.touching = 1
            shapea.actor.sprite.scale = 1.5
        if isinstance(shapeb.actor, Blob) and  isinstance(shapea.actor, Static.Static):
            shapeb.actor.touching = 1
            shapeb.actor.sprite.scale = 1.5

        aglue = hasattr(shapea, 'glue_is_active') and shapea.glue_is_active
        bglue = hasattr(shapeb, 'glue_is_active') and shapeb.glue_is_active
        if aglue or bglue:
            #print shapea.body, arbiter.contacts[0].position
            #print shapeb.body, arbiter.contacts
            anchr1 = arbiter.contacts[0].position- shapea.body.position
            anchr2 = arbiter.contacts[0].position- shapeb.body.position
            joint = pymunk.PinJoint(shapea.body, shapeb.body, anchr1, anchr2)
            joint.max_bias = 0.001
            joint.max_force = 500000
            if aglue:
                shapea.actor.glue_joints.append(joint)
                joint.glue_actors = [shapea.actor]
            if bglue:
                shapeb.actor.glue_joints.append(joint)
                if not aglue:
                    joint.glue_actors = []
                joint.glue_actors.append(shapeb.actor)
            #print "Adding joint", joint
            space.add(joint)
            self.sprite.opacity = 250
        return True


        return not ( shapea.actor == self or shapeb.actor == self)


    #def post_callback(self, space, arbiter, *args, **kwargs):
    #    self.touching = 1
    #    self.sprite.scale = 1.5
        #print "post"
    #    return True


    def activate_skill(self):
        if self.blob_type == "Blob":
            self.color = (1,0,0)
        elif self.blob_type == "JumpBlob":
            for body, shape in self.circles:
                body.apply_impulse(Vec2d(0,1000))
        elif self.blob_type == "GlueBlob":
            self.active_skill = "Glue"
            #self.color = (1,0,0)
            #self.context.space.add_collision_handler(BLOB_COLLISION_TYPE, DEFAULT_COLLISION_TYPE, self.glue_callback, None, None, None)
            #self.context.space.add_collision_handler(BLOB_COLLISION_TYPE, BLOB_COLLISION_TYPE, self.glue_callback, None, None, None)
            for c in self.circles:
                c[1].glue_is_active = True
                #print c
            pass
        elif self.blob_type == "MoveBlob":
            self.set_speed(100)

    def set_type(self, context, blobtype):
        self.switchtimer = 0.1
        print "Type from %s to %s" % (self.blob_type, blobtype)
        self.blob_type = blobtype
        if blobtype == "GlueBlob":
            self.fillcolor = (0, 0.5, 0)
            self.edgecolor = (0, 1  , 0)
            for b,s in self.circles:
                s.glue_is_active = True

            self.sprite = cocos.sprite.Sprite("Stick.png")
            self.sprite.opacity = 50
            self.add(self.sprite)
        if blobtype == "JumpBlob":
            self.fillcolor = (0.5, 0, 0)
            self.edgecolor = (1  , 0, 0)
            self.sprite = cocos.sprite.Sprite("Jump.png")
            self.add(self.sprite)
        if blobtype == "MoveBlob":
            self.fillcolor = (0.0, 0.5, 0.8)
            self.edgecolor = (0.3, 0.7, 1  )
            self.sprite = cocos.sprite.Sprite("Move.png")
            self.add(self.sprite)
            #self.set_speed(10000)

    def toggle_type(self, context):
        self.remove(self.sprite)
        if self.blob_type == "GlueBlob":
            for b,s in self.circles:
                s.glue_is_active = False
            for j in self.glue_joints:
                j.glue_actors.remove(self)
                if j.glue_actors == []:
                    context.space.remove(j)
                #print "removing joint", joint, "from", self

            self.glue_joints = []
            self.set_type(context, "JumpBlob" )
        elif self.blob_type == "JumpBlob":
            self.set_type(context, "MoveBlob")
        elif self.blob_type == "MoveBlob":
            self.set_type(context, "GlueBlob")

    def destroy(self, context):
        for s in self.springs:
            context.space.remove(s)

        for j in self.joints:
            context.space.remove(j)

        for c in self.circles:
            context.space.remove(c[0])
            context.space.remove(c[1])


        for j in self.glue_joints:
            try:
                context.space.remove(j)
            except:
                pass
